package com.microsoft.trouterclient;

/**
 * Interface for a listener that receives and responds to Trouter messages.
 */
public interface ITrouterListener {
    /**
     * Called when Trouter connection is (re-)established
     *
     * This is important mostly because of the `endpointUrl` as the URL where
     * messages are sent to by the originating service(s). Only after receiving
     * this callback and learning the Endpoint URL can the application register
     * the established channel for receiving messages.
     *
     * The Endpoint URL is intended for service-to-service communication, only
     * whitelisted authenticated services can access it and initiate Trouter
     * requests. Use `c2cUrlBase` from `connectionInfo` to obtain a similar URL
     * for client-to-client communication.
     *
     * @note Called from an internal worker thread, listeners should offload
     *       any longer processing asynchronously.
     *
     * @param endpointUrl    A URL used for routing messages from services to
     *                       this listener
     * @param connectionInfo Additional Trouter connection information
     */
    public void onTrouterConnected(String endpointUrl, ITrouterConnectionInfo connectionInfo);

    /**
     * Signals that Trouter client has become disconnected from the Trouter service.
     * This method is invoked on an internal thread. Please do not block it with any longer activities.
     */
    public void onTrouterDisconnected();

    /**
     * Called when a request is identified for this listener based on the routing path.
     * This method is invoked on an internal thread. Please do not block it with any longer activities.
     * If you don't respond the request in time, the implementation will reply with a default HTTP 504 response after
     * some timeout.
     * @param request A request containing caller's details, as received by the server.
     * @param response Response object to be filled in and sent by the client.
     */
    public void onTrouterRequest(ITrouterRequest request, ITrouterResponse response);

    /**
     * Called when a response has been sent.
     * Can be useful when you are waiting on network I/O completion.
     * This method is invoked on an internal thread. Please do not block it with any longer activities.
     * @param response Trouter response for which this callback was intended; you should not use anything else than its
     *                 getID() method at this time.
     * @param isSuccess true if the response was sent successfully, false otherwise.
     */
    public void onTrouterResponseSent(ITrouterResponse response, boolean isSuccess);
}
