package com.microsoft.trouterclient.registration;

import android.util.Log;

import java.io.IOException;
import java.io.OutputStream;
import java.net.*;
import java.util.Timer;
import java.util.TimerTask;

import com.microsoft.trouterclient.*;

class RegistrarResponse {
    private int responseCode;
    private String responseMessage;

    public RegistrarResponse(int responseCode, String responseMessage) {
        this.responseCode = responseCode;
        this.responseMessage = responseMessage;
    }

    public int getCode() {
        return responseCode;
    }

    public String getMessage() {
        return responseMessage;
    }
}

/*
 * This Java class provides functionality of automatically tracking Trouter URL updates
 * and registering them in EDF Registrat on behalf of the application
 */
public class TrouterUrlRegistrar {
    private static final String TAG = "TrouterUrlRegistrar";
    private static final String DEFAULT_REGISTRAR_URL = "https://edge.skype.com/registrar/prod/v3/registrations";
    public static final int REFRESH_REGISTRATION_IN_ADVANCE_S = 30; // 30s prior to expiration we refresh registration (same in JS client)

    private ISkypetokenProvider skypetokenProvider;
    private String registrarUrl;
    private TrouterUrlRegistrationData registrationData;
    private int maxRegistrationTtlS;

    private Object schedulingMonitor;
    // { access to the following member variables is synchronized by the schedulingMonitor
    private Timer timer;
    private String latestTrouterUrl;
    // }


    public TrouterUrlRegistrar(
        ISkypetokenProvider skypetokenProvider,      // mandatory, auth header provider
        TrouterUrlRegistrationData registrationData, // mandatory
        String registrarHostnameAndBasePath,         // optional, defaults to edge.skype.com/registrar/prod
        int maxRegistrationTtlS                      // optional, in seconds, limits registration TTL, min value is 3600
    )
    {
        // Assert mandatory parameters
        if (skypetokenProvider == null) {
            throw new IllegalArgumentException("skypetokenProvider is mandatory dependency");
        }
        if (registrationData == null) {
            throw new IllegalArgumentException("registrationData is mandatory dependency");
        }

        this.skypetokenProvider = skypetokenProvider;
        this.registrationData = registrationData;

        if (registrarHostnameAndBasePath != null && !registrarHostnameAndBasePath.isEmpty()) {
            this.registrarUrl = "https://" + registrarHostnameAndBasePath + "/v3/registrations";
            if (!isValidUrl(registrarUrl)) {
                this.registrarUrl = DEFAULT_REGISTRAR_URL;
            }
        } else {
            this.registrarUrl = DEFAULT_REGISTRAR_URL;
        }

        this.maxRegistrationTtlS = (maxRegistrationTtlS <= 3600) ? Integer.MAX_VALUE : maxRegistrationTtlS;

        this.schedulingMonitor = new Object();
        this.timer = null;
        this.latestTrouterUrl = null;
    }

    public String getRegistrationId() {
        return this.registrationData.getRegistrationId();
    }

    public void onTrouterConnected(ITrouterConnectionInfo connectionInfo) {
        if (connectionInfo.isNewEndpointUrl()) {
            int connectionTtlS = connectionInfo.getConnectionTtlSec();
            long connectionExpiration = System.currentTimeMillis() + 1000L * connectionTtlS;
            scheduleRegistration(connectionInfo.getBaseEndpointUrl(), true, connectionExpiration, 0, false, 1);
        }
    }

    private void scheduleRegistration(final String endpointUrl, boolean supposedlyNewUrl, final long trouterUrlExpiration, long delayMs, final boolean tryWithNewSkypetoken, final int tryCount) {
        // Create action that will run on timer
        TimerTask task = new TimerTask() {
            @Override
            public void run() {
                synchronized (schedulingMonitor) {
                    if (endpointUrl != latestTrouterUrl) {
                        Log.i(TAG, "Registration will be skipped because trouter URL has changed");
                        return;
                    }
                }

                int registrationTtlS = Math.min(maxRegistrationTtlS, (int) ((trouterUrlExpiration - System.currentTimeMillis()) / 1000L));
                if (registrationTtlS < REFRESH_REGISTRATION_IN_ADVANCE_S) {
                    Log.i(TAG, "Registration will be skipped due to low TTL of " + registrationTtlS + "s");
                } else {
                    boolean retry = false;
                    boolean forceTokenRefresh = false;
                    try {
                        RegistrarResponse response = postRegistration(endpointUrl, registrationTtlS, tryWithNewSkypetoken);
                        if (response.getCode() == 200 || response.getCode() == 202) {
                            long refreshRegistrationInMs = 1000L * (registrationTtlS - REFRESH_REGISTRATION_IN_ADVANCE_S);
                            Log.i(TAG, "Registration successfully POSTed, will automatically refresh in " + refreshRegistrationInMs + "ms");
                            scheduleRegistration(endpointUrl, false, trouterUrlExpiration, refreshRegistrationInMs, false, 1);
                        } else {
                            Log.e(TAG, "Registration attempt #" + tryCount + " failed with response: " + response.getCode() + " " + response.getMessage());
                            retry = true;
                            forceTokenRefresh = (response.getCode() == 401);
                        }
                    } catch (IOException e) {
                        Log.e(TAG, "Registration attempt #" + tryCount + " failed with exception: " + e.getMessage());
                        retry = true;
                    }

                    if (retry) {
                        long retryInS = Math.min((int)Math.pow(2, tryCount), 64); // 2, 4, 8, 16, 32, 64, 64, 64, 64 ... seconds
                        scheduleRegistration(endpointUrl, false, trouterUrlExpiration, 1000L * retryInS, forceTokenRefresh, tryCount + 1);
                    }
                }
            }
        };

        synchronized (schedulingMonitor)
        {
            boolean restartTimer = false;
            if (supposedlyNewUrl && endpointUrl != latestTrouterUrl) {
                // Brand new trouter URL, must register it
                latestTrouterUrl = endpointUrl;
                restartTimer = true;
            } else if (!supposedlyNewUrl && endpointUrl == latestTrouterUrl) {
                // Same trouter URL as attempted to register before, continue
                restartTimer = true;
            }

            if (restartTimer) {
                if (timer != null) {
                    try {
                        timer.cancel();
                    } catch (IllegalStateException e) {}
                }
                timer = new Timer("TrouterUrlRegistrarTimer");
                timer.schedule(task, delayMs);
            }
        }
    }

    private RegistrarResponse postRegistration(String endpointUrl, int registrationTtlS, boolean forceTokenRefresh) throws IOException {
        byte[] postData = registrationData.getRegistrationPayload(endpointUrl, registrationTtlS);

        URL url = new URL(registrarUrl);
        URLConnection con = url.openConnection();
        HttpURLConnection http = (HttpURLConnection)con;
        http.setRequestMethod("POST");
        http.setDoOutput(true);
        http.setRequestProperty("X-Skypetoken", skypetokenProvider.getSkypetoken(forceTokenRefresh));
        http.setRequestProperty("Content-Type", "application/json; charset=UTF-8");
        http.setFixedLengthStreamingMode(postData.length);
        http.connect();
        try (OutputStream os = http.getOutputStream()) {
            os.write(postData);
        }

        return new RegistrarResponse(http.getResponseCode(), http.getResponseMessage());
    }

    static boolean isValidUrl(String url) {
        try {
            new URL(url).toURI();
        } catch (MalformedURLException | URISyntaxException e) {
            return false;
        }

        return true;
    }
}
