package com.microsoft.trouterclient.registration;

import java.nio.charset.StandardCharsets;
import java.util.UUID;

/*
 * This Java class holds parameters of client Registration record
 * that can be POSTed automatically by Trouter client on behalf of the application
 */
public class TrouterUrlRegistrationData {
    private String registrationId;
    private String applicationId;
    private String platform;
    private String platformUiVersion;
    private String templateKey;
    private String productContext;
    private String context;

    public TrouterUrlRegistrationData(
        String registrationId,                  // optional, GUID is generated if null or empty
        String applicationId,                   // mandatory, should match appIds value set in PNH templates
        String platform,                        // optional
        String platformUiVersion,               // mandatory, take the value from "clientVersion" set to TrouterClientHost
        String templateKey,                     // mandatory, should match templateKeys value set in PNH templates
        String productContext,                  // optional, currently only used by TFL
        String context                          // optional
    )
    {
        // Assert mandatory parameters
        if (applicationId == null || applicationId.isEmpty()) {
            throw new IllegalArgumentException("applicationId is mandatory parameter");
        }
        if (platformUiVersion == null || platformUiVersion.isEmpty()) {
            throw new IllegalArgumentException("platformUiVersion is mandatory parameter");
        }
        if (templateKey == null || templateKey.isEmpty()) {
            throw new IllegalArgumentException("templateKey is mandatory parameter");
        }

        this.registrationId = (registrationId == null || registrationId.isEmpty())
            ? UUID.randomUUID().toString()
            : registrationId;
        this.applicationId = escapeForJson(applicationId);
        this.platform = (platform == null) ? "" : escapeForJson(platform);
        this.platformUiVersion = escapeForJson(platformUiVersion);
        this.templateKey = escapeForJson(templateKey);
        this.productContext = escapeForJson(productContext);
        this.context = (context == null) ? "" : escapeForJson(context);
    }

    String getRegistrationId() {
        return this.registrationId;
    }

    /* Creates JSON -> String -> UTF-8 byte array of
       {
         "clientDescription": {
           "appId": "...",
           "aesKey": "",
           "languageId": "en-US",
           "productContext": "...",
           "platform": "...",
           "templateKey": "...",
           "platformUIVersion": "..."
         },
         "registrationId": "534ba9cb-9ba2-4442-8ead-0356220ac27d",
         "transports": {
           "TROUTER": [
             {
               "context": "...",
               "path": "...",
               "ttl": ...
             }
           ]
         }
       }
     */
    public byte[] getRegistrationPayload(String endpointUrl, int ttlS) {
        String endpointUrlEscaped = escapeForJson(endpointUrl);

        StringBuilder sb = new StringBuilder(500); // 250 of the constant part + 250 for variables
        sb.append("{");
        sb.append("\"clientDescription\":{");
        sb.append("\"appId\":\"").append(applicationId).append("\",");
        sb.append("\"aesKey\":\"\",");
        sb.append("\"languageId\":\"en-US\",");
        if (productContext != null) {
            sb.append("\"productContext\":\"").append(productContext).append("\",");
        }
        sb.append("\"platform\":\"").append(platform).append("\",");
        sb.append("\"platformUIVersion\":\"").append(platformUiVersion).append("\",");
        sb.append("\"templateKey\":\"").append(templateKey).append("\"");
        sb.append("},");
        sb.append("\"registrationId\":\"").append(escapeForJson(registrationId)).append("\",");
        sb.append("\"nodeId\":\"\",");
        sb.append("\"transports\":{\"TROUTER\":[{");
        sb.append("\"context\":\"").append(context).append("\",");
        sb.append("\"path\":\"").append(endpointUrlEscaped).append("\",");
        sb.append("\"ttl\":").append(ttlS);
        sb.append("}]}");
        sb.append("}");

        return sb.toString().getBytes(StandardCharsets.UTF_8);
    }

    static String escapeForJson(String s) {
        if (s == null) {
            return null;
        }
        StringBuilder sb = new StringBuilder();
        escapeForJson(s, sb);
        return sb.toString();
    }

    static void escapeForJson(String s, StringBuilder sb) {
        final int len = s.length();
        for (int i = 0; i < len; i++) {
            char ch = s.charAt(i);
            switch (ch) {
                case '"':
                    sb.append("\\\"");
                    break;
                case '\\':
                    sb.append("\\\\");
                    break;
                case '\b':
                    sb.append("\\b");
                    break;
                case '\f':
                    sb.append("\\f");
                    break;
                case '\n':
                    sb.append("\\n");
                    break;
                case '\r':
                    sb.append("\\r");
                    break;
                case '\t':
                    sb.append("\\t");
                    break;
                case '/':
                    sb.append("\\/");
                    break;
                default:
                    //Reference: http://www.unicode.org/versions/Unicode5.1.0/
                    if ((ch >= '\u0000' && ch <= '\u001F') || (ch >= '\u007F' && ch <= '\u009F') || (ch >= '\u2000' && ch <= '\u20FF')) {
                        String ss = Integer.toHexString(ch);
                        sb.append("\\u");
                        for (int k = 0; k < 4 - ss.length(); k++) {
                            sb.append('0');
                        }
                        sb.append(ss.toUpperCase());
                    } else {
                        sb.append(ch);
                    }
            }
        }
    }
}
