/**
  * Copyright © Microsoft Corporation. All rights reserved.
  */

/**
  * This script loads Broadcast meeting QnA and Yammer page in teams iFrame container.
  *
  * Important: don't use double backslash comments. Since this JS is injected inside a webview, adding double backslash comments will invalidate
  * the trailing script.
  *
  */
function teamsJsClient(tabUrl, allowTopNavigation) {
  /* Validate tabUrl, must be  valid https URL */
  if (typeof tabUrl !== 'string' || !tabUrl) {
    console.error('Tab url is invalid.');
    return;
  }

  if (!tabUrl.match('^https')) {
    console.error('Tab url must use HTTPS.');
    return;
  }

  /* Validate tab frame exists */
  var tabFrame = document.getElementById('extension-tab-frame');
  tabFrame.style.background = 'black';
  var tabFrameWindow = tabFrame.contentWindow;

  if (!tabFrame || !tabFrameWindow) {
    console.error('Couldn\'t find a tab frame to host the tab.');
    return;
  }

  /* Attach a listener for message events */
  window.addEventListener('message', function (event) { processMessage(event) }, false);
  console.debug("Attached listener for message events.");

  /* Update sandbox restrictions */
  if (allowTopNavigation && tabFrame.sandbox && !tabFrame.sandbox.contains("allow-top-navigation-by-user-activation")) {
    tabFrame.sandbox.add("allow-top-navigation-by-user-activation");
  }

  /* Update the tab iframe src */
  tabFrame.src = tabUrl;

    function processMessage(event) {
      console.debug("Received an event.");
      if (!validateSdkEvent(event)) {
        return;
      }

      /* Check if Yammer event and process */
      if (isYammerEvent(event)) {
        return;
      }

      const eventData = JSON.parse(event.data);
      if (eventData.event === 'extensions.qna.updateAadToken') {
        var authToken = getAuthTokenForQna(eventData);
        if (authToken) {
          console.debug('updating token');
          postMessageToQna(event.source, authToken);
        }
      } else if(eventData.event === 'extensions.qna.notification') {
        if (!ensureNativeInterface()) {
          return;
        }
        NativeInterface.onNotificationReceived();
      } else if(eventData.event === 'extensions.qna.log') {
        /* todo handle logging event */
      } else if(eventData.event === 'extensions.qna.scenario') {
        /* todo handle scenario event */
      }
    }

  function validateSdkEvent(event) {
    if (!event || !event.data) {
      console.warn('Invalid event.');
      return false;
    }
    if (event.source !== tabFrameWindow) {
      console.warn('Received an event from an unknown frame window.');
      return false;
    }
    return true;
  }

  /* Posts Auth token to Broadcast QnA */
  function postMessageToQna(sdkWindow, authToken) {
    var response = JSON.stringify({
                                     event: 'extensions.qna.updateAadToken',
                                     data: {
                                       aadToken: `${authToken}`
                                     }
                                   });
    console.debug('posting token to QnA');
    sdkWindow.postMessage(response, '*');
  }

  /* Gets authtoken for Broadcast QnA Audience */
  function getAuthTokenForQna (eventData) {
    if (!ensureNativeInterface() || !eventData.data) {
      return undefined;
    }
    var authToken = NativeInterface.getAuthToken(eventData.data.audience);
    if (typeof authToken === 'string') {
      return authToken;
    } else {
      console.debug('Could not get valid token');
      return undefined;
    }
  }

  /* Handles if the event is yammer */
  function isYammerEvent() {
    if (!event.data.message) {
      return false;
    }
    if (event.data.message === 'yam.embed.initialLoginFailure') {
      /* we post token on when received initial login failure */
      handleYammerEmbedTokenRequest(event);
      return true;
    } else if (event.data.message === 'yam.embed.aadTokenLoginStatus') {
      /* gives login status */
      handleAadTokenLoginStatusEvent(event);
      return true;
    }
    return false;
  }

  /* handle yammer token request */
  function handleYammerEmbedTokenRequest(event) {
    console.debug('processing Yammer token request');
    var yammerUrl = new URL(tabUrl);
    if (ensureNativeInterface()) {
      var authToken = NativeInterface.getAuthToken(yammerUrl.protocol + '//' + yammerUrl.hostname);
          var response = {message: 'loginWithAadToken',
                          token: `${authToken}`};

          console.debug('posting token to Yammer');
          event.source.postMessage(response, '*');
    } else {
      console.debug('Could not get token for Yammer');
    }
  }

  /* handle yammer login status */
  function handleAadTokenLoginStatusEvent(event) {
    console.debug('received Yammer Login status');
    if (!event.data.status) {
      console.debug('Invalid login status received');
      if(ensureNativeInterface()) {
        NativeInterface.yammerLoginStatus(false);
      }
    } else if (event.data.status === 'success') {
      console.debug('Yammer login Successful');
      if (ensureNativeInterface()) {
        NativeInterface.yammerLoginStatus(true);
      }
    }else if (event.data.status === 'failure') {
      console.debug('Yammer login Unsuccessful');
      if (ensureNativeInterface()) {
        NativeInterface.yammerLoginStatus(false);
      }
    }
  }

  /* Ensures the native JS interface is defined */
  function ensureNativeInterface() {
    if (typeof NativeInterface === 'undefined') {
      console.error('Couldn\'t find a native interface to delegate the method to.');
      return false;
    }

    return true;
  }
}
