/**
  * Copyright © Microsoft Corporation. All rights reserved.
  */

/**
  * This script executes a tab built using Teams JS SDK inside an iframe host.
  *
  * Important: don't use double backslash comments. Since this JS is injected inside a webview, adding double backslash comments will invalidate
  * the trailing script.
  *
  */
function teamsJsClient(tabUrl) {
  teamsJsClient(tabUrl, false);
}

function teamsJsClient(tabUrl, allowTopNavigation) {
  /* Validate tabUrl, must be  valid https URL */
  if (typeof tabUrl !== 'string' || !tabUrl) {
    console.error('Tab url is invalid.');
    return;
  }

  if (!tabUrl.match('^https')) {
    console.error('Tab url must use HTTPS.');
    return;
  }

  /* Error strings */
  const errorStrings = {
    nativeInterfaceUndefined: 'Couldn\'t find a native interface to delegate the method to.',
    tabFrameUndefined: 'Couldn\'t find a tab frame to host the tab.',
    unhandledSDkRequest: 'Received a request that is not handled. Request: ',
    failedToGetAuthToken: 'Failed to get auth token.',
    failedToGetUser: 'Failed to get user.',
    resourceNotSpecifiedForAuth: 'Resource url not specified for authentication.getAuthToken request.',
    filePreviewArgsInvalid: 'Invalid arguments for openFilePreview message.'
  };

  /* Validate tab frame exists */
  var tabFrame = document.getElementById('extension-tab-frame');
  tabFrame.style.background = 'black';
  var tabFrameWindow = tabFrame.contentWindow;

  if (!tabFrame || !tabFrameWindow) {
    console.error(errorStrings.tabFrameUndefined);
    return;
  }

  /* Attach a listener for message events */
  window.addEventListener('message', function (event) { processMessage(event) }, false);
  console.debug("Attached listener for message events.");

  /* Update sandbox restrictions */
  if (allowTopNavigation && tabFrame.sandbox && !tabFrame.sandbox.contains("allow-top-navigation-by-user-activation")) {
    tabFrame.sandbox.add("allow-top-navigation-by-user-activation");
  }

  /* Update the tab iframe src */
  tabFrame.src = tabUrl;

  /* Processes the message recevied by the document window */
  function processMessage(event) {
    console.debug("Received an event.");
    var streamEvent = event.data;
    if (streamEvent && streamEvent.name && event.source === tabFrameWindow) {
      if (handleStreamEvent(event.source, streamEvent)) {
        return;
      }
    }

    if (!validateSdkEvent(event)) {
      return;
    }

    var request = new sdkRequest(event.data.id, event.data.func, event.data.args);
    console.debug(request.func);
    if (request.func === 'initialize') {
      postSdkResponse(event.source, request.id, ['content', 'web']);
    } else if (request.func === 'getContext') {
      postSdkResponse(event.source, request.id, getContext());
    } else if (request.func === 'authentication.getAuthToken') {
      postSdkResponse(event.source, request.id, getAuthToken(request.args));
    } else if (request.func === 'authentication.getUser') {
      postSdkResponse(event.source, request.id, getUser());
    } else if (request.func === 'openFilePreview') {
      openFilePreview(request.args);
    } else {
      console.warn(errorStrings.unhandledSDkRequest + request.func);
    }
  }

  function handleStreamEvent(eventSource, streamEvent) {
    console.debug('Stream event received');
    if (streamEvent.name === 'initializestreamconnection' || streamEvent.name === 'initialize') {
      handleInitializeConnection(eventSource, streamEvent);
    } else if (streamEvent.name === 'signinevent') {
      handleSigninEvent(streamEvent.eventData);
    } else if (streamEvent.name === 'servererror') {
      handleServerErrorEvent(streamEvent.eventData);
    } else if (streamEvent.name === 'loaddata') {
      handleLoadData(streamEvent.eventData);
    } else if (streamEvent.name === 'playerevent') {
      handlePlayerEvent(streamEvent.eventData);
    } else {
      return false;
    }
    return true;
  }

  /*Handles stream connection initialization; initialize and initializestreamconnection are the handshake events from stream embed*/
  function handleInitializeConnection(eventSource, streamEvent) {
    console.debug('handling initializestreamconnection or initialize');
    if (ensureNativeInterface()) {
      NativeInterface.onInitializeStreamConnectionReceived();
      NativeInterface.updateScenarioWithData('clientSessionId', streamEvent.clientSessionId);
    }
    /*Stream requires us to post same message with streamEvent.result set to true, to post further events*/
    streamEvent.result = true;
    if (eventSource.postMessage) {
      console.debug('Posting message back to stream with result true');
      eventSource.postMessage(JSON.stringify(streamEvent), '*');
    }
  }

  /*Handles stream signin events*/
  function handleSigninEvent(eventData) {
    console.debug('handling signinevent');
    if (eventData.eventname === 'signinsuccessful') {
      console.debug('Sign in successful');
      if (ensureNativeInterface()) {
        NativeInterface.handleSignInSuccessful();
      }
    } else if (eventData.eventname === 'signinfailed') {
      console.debug('sign in failed');
      if (ensureNativeInterface()) {
        NativeInterface.handleSignInFailed(eventData.errorcode, eventData.message);
      }
    } else if (eventData.eventname === 'signinincomplete') {
      console.debug('sign in incomplete');
      if (ensureNativeInterface()) {
        NativeInterface.updateScenarioWithData('StreamLogin', 'Incomplete');
      }
    }
  }

  /* handles server error received from stream embed */
  function handleServerErrorEvent(eventData) {
    console.debug('handling servererror');
    if (ensureNativeInterface()) {
       NativeInterface.handleServerError(eventData.errorcode, eventData.message);
    }
  }

  /*Handles data loaded status events*/
  function handleLoadData(eventData) {
    console.debug('handling loaddata');
    if (eventData.eventname === 'dataloaded') {
      console.debug(eventData.responsecode);
      if (ensureNativeInterface()) {
        NativeInterface.handleLoadData(eventData.responsecode);
      }
    }
  }

  /*Handles player events*/
  function handlePlayerEvent(eventData) {
    console.debug('handling playerevent');
    if (eventData.eventname === 'setsource') {
      console.debug('playerSessionId:' + eventData.playersessionid);
      if (ensureNativeInterface()) {
        NativeInterface.updateScenarioWithData('playerSessionId', eventData.playersessionid);
      }
    } else if (eventData.eventname === 'canplaythrough') {
      console.debug('Received canplaythrough event');
      if (ensureNativeInterface()) {
        NativeInterface.handleCanPlayThrough();
      }
    } else if (eventData.eventname === 'error') {
      console.debug('Received player error');
      if (ensureNativeInterface()) {
        NativeInterface.handlePlayerError(eventData.errorcode, eventData.message);
      }
    }
  }

  /* Validates the SDK event */
  function validateSdkEvent(event) {
    if (!event || !event.data) {
      console.warn('Invalid event.');
      return false;
    }

    if (event.source !== tabFrameWindow) {
      console.warn('Received an event from an unknown frame window.');
      return false;
    }

    if (typeof event.data.func !== 'string') {
      console.warn('Invalid request: func not specified.');
      return false;
    }

    if (typeof event.data.id !== 'number') {
      console.warn('Invalid event: id not specified.');
      return false;
    }

    if (!Array.isArray(event.data.args)) {
      console.warn('Invalid event: args not specified.');
      return false;
    }

    return true;
  }

  /* Creates a sdk request instance */
  function sdkRequest(id, func, args) {
    this.id = id;
    this.func = func;
    this.args = args;
  }

  /* Posts a response to the sdk */
  function postSdkResponse(sdkWindow, id, args) {
    var response = {
      id: id,
      args: args || []
    };

    sdkWindow.postMessage(response, '*');
  }

  /* Get current tab context */
  function getContext() {
    if (!ensureNativeInterface()) {
      return null;
    }

    return [JSON.parse(NativeInterface.getContext())];
  }

  /* Gets auth token for the specified resource in the arguments */
  function getAuthToken(args) {
    if (!ensureNativeInterface()) {
      return [false, errorStrings.nativeInterfaceUndefined];
    }

    if (args.length == 0 || !Array.isArray(args[0])
      || args[0].length == 0 || typeof args[0][0] !== 'string') {
      return [false, errorStrings.resourceNotSpecifiedForAuth]
    }
console.debug(args);
    var authToken = NativeInterface.getAuthToken(args[0][0]);
    if (typeof authToken === 'string') {
      return [true, authToken];
    } else {
      return [false, errorStrings.failedToGetAuthToken];
    }
  }

  function getUser() {
    if (!ensureNativeInterface()) {
      return [false, errorStrings.nativeInterfaceUndefined];
    }

    var userString = NativeInterface.getUser();
    console.debug('user string is ' + userString);
    if (typeof userString === 'string') {
      return [true, [JSON.parse(NativeInterface.getUser())]];
    } else {
      return [false, errorStrings.failedToGetUser];
    }
  }

  /* Opens preview for the specified file in the arguments */
  function openFilePreview(args) {
    if (!ensureNativeInterface()) {
      return [false, errorStrings.nativeInterfaceUndefined];
    }

    if (args == null || args.length < 6) {
      return [false, errorStrings.filePreviewArgsInvalid]
    }

    var fileTitle = args[1];
    var fileType = args[3];
    var fileUrl = args[4];
    var fileDownloadUrl = args[5];
    NativeInterface.openFilePreview(fileTitle, fileType, fileUrl, fileDownloadUrl);
  }

  /* Ensures the native JS interface is defined */
  function ensureNativeInterface() {
    if (typeof NativeInterface === 'undefined') {
      console.error(errorStrings.nativeInterfaceUndefined);
      return false;
    }

    return true;
  }
}